
#include <iostream>
#include <fstream>
#include <sstream>
#include <memory>
#include <string>

#include <grpcpp/grpcpp.h>
#include "turbo.grpc.pb.h"

using namespace std;
using namespace grpc;
using namespace Turbo;

class GreeterClient 
{
	public:
		GreeterClient(shared_ptr<Channel> channel)
		: stub_(Greeter::NewStub(channel)) 
		{
		}

		// Assembles the client's payload, sends it and presents the response back
		// from the server.
		void Ping() 
		{

			const auto send_filename = "/bin/ls";
			ifstream fin(send_filename, ios::binary);
			ostringstream ostrm;
			ostrm << fin.rdbuf();
			const auto send_contents = ostrm.str();

			cout << "Client sending file " << send_filename << ": " << endl;
			cout << send_contents.substr(0,10) << endl;

			// Data we are sending to the server.
			TurboRequest_t request;
			request.set_orig_file_name(send_filename);
			request.set_orig_contents(send_contents);

			// Container for the data we expect from the server.
			TurboReply_t reply;

			// Context for the client. It could be used to convey extra information to
			// the server and/or tweak certain RPC behaviors.
			ClientContext context;

			// The actual RPC.
			Status status = stub_->Ping(&context, request, &reply);

			// Act upon its status.
			if (status.ok()) 
			{
				cout << "Client received file " << reply.zafl_file_name() << endl;;
				cout << reply.zafl_contents().substr(0,10) << endl;

				ofstream fout("client.cat", ios::binary);
				fout << reply.zafl_contents();
			} 
			else 
			{
				cout << status.error_code() << ": " << status.error_message() << endl;
			}
		}

	private:
		unique_ptr<Greeter::Stub> stub_;
};

int main(int argc, char **argv) 
{
	// Instantiate the client. It requires a channel, out of which the actual RPCs
	// are created. This channel models a connection to an endpoint (in this case,
	// localhost at port 50051). We indicate that the channel isn't authenticated
	// (use of InsecureChannelCredentials()).
	GreeterClient greeter(grpc::CreateChannel(
		"localhost:50051", InsecureChannelCredentials()));
	greeter.Ping();
	return 0;
}
